@@ -1,32 +1,6 @@
 # -*- coding: utf8 -*-
-"""This is just here to make pylint happy """
-# Copyright (C) 2002-2010  Tobias Klausmann
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.        See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330 Boston, MA
-
-import fpformat
-import math
-import re
-import urllib2
-
-__author__ = "klausman-pymetar@schwarzvogel.de"
-
-__version__ = "0.16"
-__revision__ = "$Rev: 106 $"[6:-2]
-
-__doc__ = """Pymetar v%s (c) 2002-2010 Tobias Klausmann
+# pylint: disable-msg=C0103 # Disable naming style messages
+"""Pymetar (c) 2002-2012 Tobias Klausmann
 
 Pymetar is a python module and command line tool designed to fetch Metar
 reports from the NOAA (http://www.noaa.gov) and allow access to the
@@ -44,14 +18,23 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."""
 
-Please e-mail bugs to: %s""" % (__version__, __author__)
+import fpformat
+import math
+import re
+import urllib2
+
+__author__ = "klausman-pymetar@schwarzvogel.de"
+
+__version__ = "0.19"
+__revision__ = "$Rev$"[6:-2]
 
 
-CLOUD_RE_STR = r"^(CAVOK|CLR|SKC|BKN|SCT|FEW|OVC|NSC)([0-9]{3})(TCU|CU|CB|SC|CBMAM|ACC|SCSL|CCSL|ACSL)?$"
-CTYPE_RE_STR = r"^(TCU|CU|CB|SC|CBMAM|ACC|SCSL|CCSL|ACSL)?$"
-COND_RE_STR  = r"^(-|\\+)?(VC|MI|BC|PR|TS|BL|SH|DR|FZ)?(DZ|RA|SN|SG|IC|PE|GR|GS|UP|BR|FG|FU|VA|SA|HZ|PY|DU|SQ|SS|DS|PO|\\+?FC)$"
+CLOUD_RE_STR = (r"^(CAVOK|CLR|SKC|BKN|SCT|FEW|OVC|NSC)([0-9]{3})?"
+                r"(TCU|CU|CB|SC|CBMAM|ACC|SCSL|CCSL|ACSL)?$")
+COND_RE_STR  = (r"^(-|\\+)?(VC|MI|BC|PR|TS|BL|SH|DR|FZ)?(DZ|RA|SN|SG|IC|PE|"
+                r"GR|GS|UP|BR|FG|FU|VA|SA|HZ|PY|DU|SQ|SS|DS|PO|\\+?FC)$")
 
 class EmptyReportException(Exception):
     """This gets thrown when the ReportParser gets fed an empty report"""
@@ -69,7 +52,7 @@ class NetworkException(Exception):
 # It seems the NOAA doesn't want to return plain text, but considering the
 # format of their response, this is not to save bandwidth :-)
 
-_WeatherConditions = {
+_WEATHER_CONDITIONS = {
                       "DZ" : ("Drizzle", "rain", {
                                "" :   "Moderate drizzle",
                                "-" :  "Light drizzle",
@@ -357,7 +340,7 @@ _WeatherConditions = {
                              }),
                     }
 
-CLOUDTYPES = { 
+CLOUDTYPES = {
                "ACC": "altocumulus castellanus",
                "ACSL": "standing lenticular altocumulus",
                "CB": "cumulonimbus",
@@ -376,56 +359,49 @@ def metar_to_iso8601(metardate) :
         (date, hour) = metardate.split()[:2]
         (year, month, day) = date.split('.')
         # assuming tz is always 'UTC', aka 'Z'
-        return "%s-%s-%s %s:%s:00Z" % \
-            (year, month, day, hour[:2], hour[2:4])
+        return ("%s-%s-%s %s:%s:00Z" %
+                (year, month, day, hour[:2], hour[2:4]))
 
-def parseLatLong(latlong):
+def _parse_lat_long(latlong):
     """
-    Parse Lat or Long in METAR notation into float values. N and E 
-    are +, S and W are -. Expects one positional string and returns 
+    Parse Lat or Long in METAR notation into float values. N and E
+    are +, S and W are -. Expects one positional string and returns
     one float value.
     """
     # I know, I could invert this if and put
     # the rest of the function into its block,
     # but I find it to be more readable this way
-    if latlong is None: 
+    if latlong is None:
         return None
 
-    s = latlong.upper().strip()
-    elms = s.split('-')
-    ud = elms[-1][-1]
-    elms[-1] = elms[-1][:-1]
-    elms = [int(i) for i in elms]
+    cap_inp = latlong.upper().strip()
+    elements = cap_inp.split('-')
+    # Extract N/S/E/W
+    compass_dir = elements[-1][-1]
+    # get rid of compass direction
+    elements[-1] = elements[-1][:-1]
+    elements = [int(i) for i in elements]
     coords = 0.0
-    elen = len(elms)
+    elen = len(elements)
     if elen > 2:
-        coords = coords + float(elms[2])/3600.0
+        coords = coords + float(elements[2])/3600.0
 
     if elen > 1:
-        coords = coords + float(elms[1])/60.0
+        coords = coords + float(elements[1])/60.0
 
-    coords = coords + float(elms[0])
+    coords = coords + float(elements[0])
 
-    if ud in ('W', 'S'):
+    if compass_dir in ('W', 'S'):
         coords = -1.0*coords
 
-    f, i = math.modf(coords)
-
-    if elen > 2:
-        f = float(fpformat.sci(f, 4))
-    elif elen > 1:
-        f = float(fpformat.sci(f, 2))
-    else:
-        f = 0.0
-
-    return f+i
+    return coords
 
 class WeatherReport:
     """Incorporates both the unparsed textual representation of the
-    weather report and the parsed values as soon as they are filled 
+    weather report and the parsed values as soon as they are filled
     in by ReportParser."""
 
-    def _ClearAllFields(self):
+    def _clearallfields(self):
         """Clear all fields values."""
         # until finished, report is invalid
         self.valid = 0
@@ -442,6 +418,7 @@ class WeatherReport:
         self.dewpf = None
         self.humid = None
         self.press = None
+        self.pressmmHg = None
         self.code = None
         self.weather = None
         self.sky = None
@@ -466,7 +443,7 @@ class WeatherReport:
 
     def __init__(self, MetarStationCode = None):
         """Clear all fields and fill in wanted station id."""
-        self._ClearAllFields()
+        self._clearallfields()
         self.givenstationid = MetarStationCode
 
     def getFullReport(self):
@@ -523,7 +500,7 @@ class WeatherReport:
         Return the wind speed in knots
         """
         if self.windspeed is not None:
-            return self.windspeedknots
+            return self.windspeed * 1.94384449
 
     def getWindDirection(self):
         """
@@ -564,6 +541,12 @@ class WeatherReport:
         """
         return self.press
 
+    def getPressuremmHg(self):
+        """
+        Return pressure in mmHg.
+        """
+        return self.pressmmHg
+
     def getRawMetarCode(self):
         """
         Return the encoded weather report.
@@ -603,9 +586,9 @@ class WeatherReport:
     def getCycle(self):
         """
         Return cycle value.
-        The cycle value is not the frequency or delay between 
-        observations but the "time slot" in which the observation was made. 
-        There are 24 cycle slots every day which usually last from N:45 to 
+        The cycle value is not the frequency or delay between
+        observations but the "time slot" in which the observation was made.
+        There are 24 cycle slots every day which usually last from N:45 to
         N+1:45. The cycle from 23:45 to 0:45 is cycle 0.
         """
         return self.cycle
@@ -613,19 +596,19 @@ class WeatherReport:
     def getStationPosition(self):
         """
         Return latitude, longitude and altitude above sea level of station
-        as a tuple. Some stations don't deliver altitude, for those, None 
+        as a tuple. Some stations don't deliver altitude, for those, None
         is returned as altitude.  The lat/longs are expressed as follows:
         xx-yyd
         where xx is degrees, yy minutes and d the direction.
-        Thus 51-14N means 51 degrees, 14 minutes north.  d may take the 
-        values N, S for latitues and W, E for longitudes. Latitude and 
-        Longitude may include seconds.  Altitude is always given as meters 
+        Thus 51-14N means 51 degrees, 14 minutes north.  d may take the
+        values N, S for latitues and W, E for longitudes. Latitude and
+        Longitude may include seconds.  Altitude is always given as meters
         above sea level, including a trailing M.
         Schipohl Int. Airport Amsterdam has, for example:
         ('52-18N', '004-46E', '-2M')
         Moenchengladbach (where I live):
         ('51-14N', '063-03E', None)
-        If you need lat and long as float values, look at 
+        If you need lat and long as float values, look at
         getStationPositionFloat() instead
         """
         # convert self.altitude to string for consistency
@@ -633,7 +616,7 @@ class WeatherReport:
 
     def getStationPositionFloat(self):
         """
-        Return latitude and longitude as float values in a 
+        Return latitude and longitude as float values in a
         tuple (lat, long, alt).
         """
         return (self.latf, self.longf, self.altitude)
@@ -684,7 +667,7 @@ class WeatherReport:
 
     def getTime(self):
         """
-        Return the time when the observation was made.  Note that this 
+        Return the time when the observation was made.  Note that this
         is *not* the time when the report was fetched by us
         Format:  YYYY.MM.DD HHMM UTC
         Example: 2002.04.01 1020 UTC
@@ -700,7 +683,7 @@ class WeatherReport:
 
     def getPixmap(self):
         """
-        Return a suggested pixmap name, without extension, depending on 
+        Return a suggested pixmap name, without extension, depending on
         current weather.
         """
         return self.pixmap
@@ -723,26 +706,28 @@ class WeatherReport:
         """
         Return wind chill in degrees Celsius
         """
-        # I've had two choices here: simply convert the Fahrenheit windchill (see below)
-        # or calculate it seperately if needed. I chose the latter for accuracy reasons
-        if self.w_chill ==  None:
-            if  self.temp and self.temp < 10 and \
-                self.windspeed and (self.windspeed*3.6) > 4.8:
-                self.w_chill = 13.12 + 0.6215*self.temp - 11.37*(self.windspeed*3.6)**0.16\
-                                 + 0.3965*self.temp*(self.windspeed*3.6)**0.16
+        # http://en.wikipedia.org/wiki/Wind_chill - North American wind chill
+        # index
+        if self.w_chill is None:
+            if (self.temp and self.temp <= 10 and
+                self.windspeed and (self.windspeed*3.6) > 4.8):
+                self.w_chill = (13.12 + 0.6215*self.temp -
+                                11.37*(self.windspeed*3.6)**0.16 +
+                                0.3965*self.temp*(self.windspeed*3.6)**0.16)
         return self.w_chill
 
     def getWindchillF(self):
         """
         Return wind chill in degrees Fahrenheit
         """
-        # This code is derived from NOAA's test on the topic, put into code (and donated)
-        # by Alexander Voronin
-        if self.w_chillf == None:
-            if  self.tempf and self.tempf <= 50 and \
-                self.windspeedmph and self.windspeedmph >= 3:
-                self.w_chillf = 35.74 + 0.6215*self.tempf - 35.75*self.windspeedmph**0.16 + \
-                                0.4275*self.tempf*self.windspeedmph**0.16
+        # http://en.wikipedia.org/wiki/Wind_chill - North American wind chill
+        # index
+        if self.w_chillf is None:
+            if (self.tempf and self.tempf <= 50 and
+                self.windspeedmph and self.windspeedmph >= 3):
+                self.w_chillf = (35.74 + 0.6215*self.tempf -
+                                 35.75*self.windspeedmph**0.16 +
+                                 0.4275*self.tempf*self.windspeedmph**0.16)
             else:
                 self.w_chillf = self.tempf
 
@@ -757,7 +742,7 @@ class WeatherReport:
 
 
 class ReportParser:
-    """Parse raw METAR data from a WeatherReport object into actual 
+    """Parse raw METAR data from a WeatherReport object into actual
     values and return the object with the values filled in."""
 
     def __init__(self, MetarReport = None):
@@ -768,7 +753,7 @@ class ReportParser:
         """
         Extract cloud information. Return None or a tuple (sky type as a
         string of text, cloud type (if any)  and suggested pixmap name)
-        """   
+        """
         matches = self.match_WeatherPart(CLOUD_RE_STR)
         skytype = None
         ctype = None
@@ -777,20 +762,20 @@ class ReportParser:
             if wcloud is not None:
                 stype = wcloud[:3]
                 if stype in ("CLR", "SKC", "CAV", "NSC"):
-                    skytype="Clear sky"
-                    pixmap="sun"
+                    skytype = "Clear sky"
+                    pixmap = "sun"
                 elif stype == "BKN" :
-                    skytype="Broken clouds"
-                    pixmap="suncloud"
+                    skytype = "Broken clouds"
+                    pixmap = "suncloud"
                 elif stype == "SCT" :
-                    skytype="Scattered clouds"
-                    pixmap="suncloud"
+                    skytype = "Scattered clouds"
+                    pixmap = "suncloud"
                 elif stype == "FEW" :
-                    skytype="Few clouds"
-                    pixmap="suncloud"
+                    skytype = "Few clouds"
+                    pixmap = "suncloud"
                 elif stype == "OVC" :
-                    skytype="Overcast"
-                    pixmap="cloud"
+                    skytype = "Overcast"
+                    pixmap = "cloud"
                 if ctype == None:
                     ctype = CLOUDTYPES.get(wcloud[6:], None)
 
@@ -798,25 +783,25 @@ class ReportParser:
 
     def extractSkyConditions(self) :
         """
-        Extract sky condition information from the encoded report. Return 
-        a tuple containing the description of the sky conditions as a 
-        string and a suggested pixmap name for an icon representing said 
+        Extract sky condition information from the encoded report. Return
+        a tuple containing the description of the sky conditions as a
+        string and a suggested pixmap name for an icon representing said
         sky condition.
         """
-        matches = self.match_WeatherPart(CTYPE_RE_STR)
+        matches = self.match_WeatherPart(COND_RE_STR)
         for wcond in matches:
-            if (len(wcond)>3) and (wcond.startswith('+') \
-                or wcond.startswith('-')) :
+            if ((len(wcond)>3) and
+                (wcond.startswith('+') or wcond.startswith('-'))):
                 wcond = wcond[1:]
-            if wcond.startswith('+') or wcond.startswith('-') :
+            if wcond.startswith('+') or wcond.startswith('-'):
                 pphen = 1
-            elif len(wcond) < 4 :
+            elif len(wcond) < 4:
                 pphen = 0
             else :
                 pphen = 2
             squal = wcond[:pphen]
             sphen = wcond[pphen : pphen + 4]
-            phenomenon = _WeatherConditions.get(sphen, None)
+            phenomenon = _WEATHER_CONDITIONS.get(sphen, None)
             if phenomenon is not None :
                 (name, pixmap, phenomenon) = phenomenon
                 pheninfo = phenomenon.get(squal, name)
@@ -828,28 +813,28 @@ class ReportParser:
 
     def match_WeatherPart(self, regexp) :
         """
-        Return the matching part of the encoded Metar report.  
-        regexp: the regexp needed to extract this part.  
-        Return the first matching string or None.  
-        WARNING: Some Metar reports may contain several matching 
+        Return the matching part of the encoded Metar report.
+        regexp: the regexp needed to extract this part.
+        Return the first matching string or None.
+        WARNING: Some Metar reports may contain several matching
         strings, only the first one is taken into account!
         """
-        matches=[]
+        matches = []
         if self.Report.code is not None :
-            rg = re.compile(regexp)
+            myre = re.compile(regexp)
             for wpart in self.Report.getRawMetarCode().split() :
-                match = rg.match(wpart)
+                match = myre.match(wpart)
                 if match:
                     matches.append(match.string[match.start(0) : match.end(0)])
         return matches
 
     def ParseReport(self, MetarReport = None):
-        """Take report with raw info only and return it with in 
+        """Take report with raw info only and return it with in
         parsed values filled in. Note: This function edits the
         WeatherReport object you supply!"""
         if self.Report is None and MetarReport is None:
-            raise EmptyReportException, \
-                "No report given on init and ParseReport()."
+            raise EmptyReportException(
+                "No report given on init and ParseReport().")
         elif MetarReport is not None:
             self.Report = MetarReport
 
@@ -870,7 +855,7 @@ class ReportParser:
             if header.find("("+self.Report.givenstationid+")") != -1:
                 id_offset = header.find("("+self.Report.givenstationid+")")
                 loc = data[:id_offset]
-                p = data[id_offset:]
+                coords = data[id_offset:]
                 try:
                     loc = loc.strip()
                     rloc = loc[::-1]
@@ -878,54 +863,54 @@ class ReportParser:
                 except ValueError:
                     rcity = ""
                     rcoun = ""
-                    p = data
+                    coords = data
                 try:
-                    lat, lng, ht = p.split()[1:4]
-                    ht = int(ht[:-1]) # cut off 'M' for meters
+                    lat, lng, alt = coords.split()[1:4]
+                    alt = int(alt[:-1]) # cut off 'M' for meters
                 except ValueError:
-                    (lat, lng) = p.split()[1:3]
-                    ht = None
+                    (lat, lng) = coords.split()[1:3]
+                    alt = None
                 # A few jokers out there think O==0
                 if "O" in lat:
-                    lat=lat.replace("O", "0")
+                    lat = lat.replace("O", "0")
                 if "O" in lng:
-                    lng=lng.replace("O", "0")
+                    lng = lng.replace("O", "0")
 
                 self.Report.stat_city = rcity.strip()[::-1]
                 self.Report.stat_country = rcoun.strip()[::-1]
                 self.Report.fulln = loc
                 self.Report.latitude = lat
                 self.Report.longitude = lng
-                self.Report.latf = parseLatLong(lat)
-                self.Report.longf = parseLatLong(lng)
-                self.Report.altitude = ht
+                self.Report.latf = _parse_lat_long(lat)
+                self.Report.longf = _parse_lat_long(lng)
+                self.Report.altitude = alt
 
             # The line containing date and time of the report
             # We have to make sure that the station ID is *not*
             # in this line to avoid trying to parse the ob: line
-            elif ((data.find(" UTC")) != -1 and \
-                    (data.find(self.Report.givenstationid)) == -1):
-                rt = data.split("/")[1]
-                self.Report.rtime = rt.strip()
+            elif ((data.find(" UTC")) != -1 and
+                  (data.find(self.Report.givenstationid)) == -1):
+                rtime = data.split("/")[1]
+                self.Report.rtime = rtime.strip()
 
             # temperature
 
             elif (header == "Temperature"):
-                f, c = data.split(None, 3)[0:3:2]
-                self.Report.tempf = float(f)
+                fnht, cels = data.split(None, 3)[0:3:2]
+                self.Report.tempf = float(fnht)
                 # The string we have split is "(NN C)", hence the slice
-                self.Report.temp = float(c[1:])
-            
+                self.Report.temp = float(cels[1:])
+
             # wind chill
 
             elif (header == "Windchill"):
-                f, c = data.split(None, 3)[0:3:2]
-                self.Report.w_chillf = float(f)
+                fnht, cels = data.split(None, 3)[0:3:2]
+                self.Report.w_chillf = float(fnht)
                 # The string we have split is "(NN C)", hence the slice
-                self.Report.w_chill = float(c[1:])
+                self.Report.w_chill = float(cels[1:])
 
             # wind dir and speed
-            
+
             elif (header == "Wind"):
                 if (data.find("Calm") != -1):
                     self.Report.windspeed = 0.0
@@ -942,12 +927,10 @@ class ReportParser:
                     self.Report.windcomp = None
                 else:
                     fields = data.split(" ", 9)[0:9]
-                    f = fields[0]
                     comp = fields[2]
                     deg = fields[3]
                     speed = fields[6]
                     speedkt = fields[8][1:]
-                    del fields
                     self.Report.winddir = int(deg[1:])
                     self.Report.windcomp = comp.strip()
                     self.Report.windspeed = (float(speed)*0.44704)
@@ -957,36 +940,38 @@ class ReportParser:
             # visibility
 
             elif (header == "Visibility"):
-                for d in data.split():
+                for visgroup in data.split():
                     try:
-                        self.Report.vis = float(d)*1.609344
+                        self.Report.vis = float(visgroup)*1.609344
                         break
                     except ValueError:
                         self.Report.vis = None
                         break
 
             # dew point
-            
+
             elif (header == "Dew Point"):
-                f, c = data.split(None, 3)[0:3:2]
-                self.Report.dewpf = float(f)
+                fnht, cels = data.split(None, 3)[0:3:2]
+                self.Report.dewpf = float(fnht)
                 # The string we have split is "(NN C)", hence the slice
-                self.Report.dewp = float(c[1:])
+                self.Report.dewp = float(cels[1:])
 
             # humidity
-             
+
             elif (header == "Relative Humidity"):
                 h = data.split("%", 1)[0]
                 self.Report.humid = int(h)
 
             # pressure
-            
+
             elif (header == "Pressure (altimeter)"):
-                p = data.split(" ", 1)[0]
-                self.Report.press = (float(p)*33.863886)
+                press = data.split(" ", 1)[0]
+                self.Report.press = (float(press)*33.863886)
+                # 1 in = 25.4 mm => 1 inHg = 25.4 mmHg
+                self.Report.pressmmHg = (float(press)*25.4000)
 
             # shot weather desc. ("rain", "mist", ...)
-            
+
             elif (header == "Weather"):
                 self.Report.weather = data
 
@@ -996,14 +981,20 @@ class ReportParser:
                 self.Report.sky = data
 
             # the encoded report itself
-            
+
             elif (header == "ob"):
                 self.Report.code = data.strip()
 
             # the cycle value ("time slot")
 
             elif (header == "cycle"):
-                self.Report.cycle = int(data)
+                try:
+                    self.Report.cycle = int(data)
+                except ValueError:
+                    # cycle value is missing or garbled, assume cycle 0
+                    # TODO: parse the date/time header if it isn't too involved
+                    self.Report.cycle = 0
+
 
         # cloud info
         cloudinfo = self.extractCloudInformation()
@@ -1037,10 +1028,10 @@ class ReportParser:
 
 class ReportFetcher:
     """Fetches a report from a given METAR id, optionally taking into
-       account a different baseurl and using environment var-specified 
+       account a different baseurl and using environment var-specified
        proxies."""
 
-    def __init__(self, MetarStationCode = None, baseurl = \
+    def __init__(self, MetarStationCode = None, baseurl =
         "http://weather.noaa.gov/pub/data/observations/metar/decoded/"):
         """Set stationid attribute and base URL to fetch report from"""
         self.stationid = MetarStationCode
@@ -1068,18 +1059,18 @@ class ReportFetcher:
         protocol://user:password@host.name.tld:port/
         is expected, for example:
         http://squid.somenet.com:3128/
-        If no proxy is specified, the environment variable http_proxy 
+        If no proxy is specified, the environment variable http_proxy
         is inspected. If it isn't set, a direct connection is tried.
         """
         if self.stationid is None and StationCode is None:
-            raise EmptyIDException, \
-                "No ID given on init and FetchReport()."
+            raise EmptyIDException(
+                "No ID given on init and FetchReport().")
         elif StationCode is not None:
             self.stationid = StationCode
 
         self.stationid = self.stationid.upper()
         self.reporturl = "%s%s.TXT" % (self.baseurl, self.stationid)
-        
+
         if proxy:
             p_dict = {'http': proxy}
             p_handler = urllib2.ProxyHandler(p_dict)
@@ -1099,14 +1090,14 @@ class ReportFetcher:
 
         if fn.info().status:
             raise NetworkException, "Could not fetch METAR report"
-            
+
         report = WeatherReport(self.stationid)
         report.reporturl = self.reporturl
         report.fullreport = self.fullreport
         self.report = report # Caching it for GetReport()
 
         return report
-        
+
     def GetReport(self):
         """Get a previously fetched report again"""
         return self.report
